/*=============================================================================
// File Name:   W25Qxx.c
// Author:      Wood Wang
// Date:        2020/03/14
// Version:     v1.0

=============================================================================*/
/* Includes -----------------------------------------------------------------*/

#include "W25Qxx.h"



/********************************* Ŷ ************************************
*    PA3  <===========>  SCS
*    PA0  <===========>  SCK
*    PA1  <===========>  DI/MOSI
*    PA2  <===========>  DO/MISO
*******************************************************************************/
#define  SPI1_CS_LOW()        GPIOA_ResetBits( GPIO_Pin_3 )
#define  SPI1_CS_HIGH()       GPIOA_SetBits( GPIO_Pin_3 )


#define _spi_read_write(d)      SPI1_MasterSendRecvByte(d)
//============================================================================
//
//
//
//============================================================================
void Flash_SPI_Configuration(void)
{
	GPIOA_SetBits( GPIO_Pin_0|GPIO_Pin_1|GPIO_Pin_3 );
	GPIOA_ModeCfg( GPIO_Pin_0|GPIO_Pin_1|GPIO_Pin_3, GPIO_ModeOut_PP_5mA );	// MOSI/SCK/CS
	GPIOA_SetBits( GPIO_Pin_2 );
	GPIOA_ModeCfg( GPIO_Pin_2, GPIO_ModeIN_PU );	// MISO	
	
    SPI1_MasterDefInit();
    R8_SPI1_CTRL_MOD = RB_SPI_MOSI_OE | RB_SPI_SCK_OE | RB_SPI_MISO_OE ;
    SPI1_DataMode( Mode3_HighBitINFront );
    SPI1_CLKCfg( 4 );

    SPI1_CS_HIGH(); 
}

//============================================================================
//
//
//
//============================================================================
//дĴ
void _RegWrite(uint8_t addr,uint8_t dat)
{
	SPI1_CS_LOW();
    
	_spi_read_write(addr);
    _spi_read_write(dat);
    
	SPI1_CS_HIGH();
}

//Ĵ
uint8_t _RegRead(uint8_t addr)
{
	uint8_t		buff[1];

	SPI1_CS_LOW();
    
	_spi_read_write(addr);
    
    buff[0] = _spi_read_write(0xff);
    
	SPI1_CS_HIGH();
    
	return buff[0];
}

////////////////////////////////////////////////////////////////////////////
//ӿ
uint16_t W25QXX_ReadID(void)
{
	uint8_t		buff[4];
	
	buff[0]= 0x90;
	buff[1] =0x00;
	buff[2] =0x00;
	buff[3] =0x00;	

	SPI1_CS_LOW();
    
	_spi_read_write(buff[0]);
    _spi_read_write(buff[1]);
    _spi_read_write(buff[2]);
    _spi_read_write(buff[3]);
    
	buff[0] = _spi_read_write(0xff);
	buff[1] = _spi_read_write(0xff);    

	SPI1_CS_HIGH();
        
	return ((uint16_t)buff[0]<<8)|buff[1];
} 

//״̬
uint8_t W25QXX_ReadSR(void)   
{     
	return _RegRead(W25X_ReadStatusReg);   
} 
//д״̬
void W25QXX_WriteSR(uint8_t sr)   
{
	_RegWrite(W25X_WriteStatusReg,sr); 	      
} 

void W25QXX_WaitBusy(void)
{ 
	while((W25QXX_ReadSR()&0x01)==0x01);  		//
}

//дʹ>0/дֹ=0
void W25QXX_WriteEnable(uint8_t Enable)   
{
	uint8_t		buff[1];
    
	if(Enable)	buff[0] = W25X_WriteEnable;
	else		buff[0] = W25X_WriteDisable;
	
	SPI1_CS_LOW();
    
	_spi_read_write(buff[0]);

	SPI1_CS_HIGH(); 	      
} 

//ģʽ
void W25QXX_PowerDown(void)   
{ 
	uint8_t		buff[1];
	
	buff[0] = W25X_PowerDown;
  
 	SPI1_CS_LOW();    
	_spi_read_write(buff[0]); 
	SPI1_CS_HIGH();    

} 

//
void W25QXX_WAKEUP(void)   
{
	uint8_t		buff[1];
	
	buff[0] = W25X_ReleasePowerDown;
	
 	SPI1_CS_LOW();    
	_spi_read_write(buff[0]); 
	SPI1_CS_HIGH();   

} 


//оƬȴʱ䳬
void W25QXX_EraseChip(void)   
{
	uint8_t		buff[1];
	
	W25QXX_WriteEnable(1);
	W25QXX_WaitBusy();
	
	buff[0] = W25X_ChipErase;
	
 	SPI1_CS_LOW();    
	_spi_read_write(buff[0]); 
	SPI1_CS_HIGH(); 
    
	W25QXX_WaitBusy();   				   		//
} 

//64K
void W25QXX_EraseBlock(uint32_t Addr)   
{  
	uint8_t		buff[5];
	buff[0]= W25X_BlockErase;
	buff[1]= (Addr)>>16;
	buff[2]= (Addr)>>8;
	buff[3]= (Addr)&0xff;
	buff[4]= 0;
	
	W25QXX_WriteEnable(1);
	W25QXX_WaitBusy();	
	
	SPI1_CS_LOW();
    
	_spi_read_write(buff[0]);
    _spi_read_write(buff[1]);
    _spi_read_write(buff[2]);
    _spi_read_write(buff[3]);

	SPI1_CS_HIGH();
    
	W25QXX_WaitBusy();					   //
} 

//4K
void W25QXX_EraseSector(uint32_t Addr)   
{  
	uint8_t		buff[5];
	buff[0]= W25X_SectorErase;
	buff[1]= (Addr)>>16;
	buff[2]= (Addr)>>8;
	buff[3]= (Addr)&0xff;
	buff[4]= 0;
	
	W25QXX_WriteEnable(1);
	W25QXX_WaitBusy();	
	
	SPI1_CS_LOW();
    
	_spi_read_write(buff[0]);
    _spi_read_write(buff[1]);
    _spi_read_write(buff[2]);
    _spi_read_write(buff[3]);   

	SPI1_CS_HIGH();

	W25QXX_WaitBusy();					   //
} 

//
void W25QXX_Read(uint32_t Addr,uint8_t* pBuffer,uint32_t Len)   
{
	uint8_t		buff[5];
    uint32_t    l;
    
//    printf("W25QXX_Read 0x%X:0x%X.\n",Addr,Len);
    
	buff[0]= W25X_ReadData;
	buff[1]= (Addr)>>16;
	buff[2]= (Addr)>>8;
	buff[3]= (Addr)&0xff;
	buff[4]= 0;

	SPI1_CS_LOW();
    
	_spi_read_write(buff[0]);
    _spi_read_write(buff[1]);
    _spi_read_write(buff[2]);
    _spi_read_write(buff[3]);   

    for(l=0;l<Len;l++)
    {
        *pBuffer ++ = _spi_read_write(0xff);
    }
	SPI1_CS_HIGH();	
}

//д
void W25QXX_WritePage(uint32_t Addr,uint8_t* pBuffer,uint32_t Len)
{
	uint8_t		buff[5];
    uint32_t    l;

//                uint8_t * p;
//                uint32_t i,j;
 //               printf("W25QXX_WritePage 0x%X:0x%X.\n",Addr,Len);
//                p = pBuffer;
//                for(i=0; i!=Len>>4; i++)
//                {
//                    for(j=0; j!=16; j++)
//                    {
//                        printf("%02X ",*p++); 
//                    }
//                    printf("\n"); 
//                }

                
	buff[0]= W25X_PageProgram;
	buff[1]= (Addr)>>16;
	buff[2]= (Addr)>>8;
	buff[3]= (Addr)&0xff;
	buff[4]= 0;
	
	W25QXX_WriteEnable(1);

	SPI1_CS_LOW();
    
	_spi_read_write(buff[0]);
    _spi_read_write(buff[1]);
    _spi_read_write(buff[2]);
    _spi_read_write(buff[3]);   

    for(l=0;l<Len;l++)
    {
         _spi_read_write(*pBuffer ++);
    }
	SPI1_CS_HIGH();	

	W25QXX_WaitBusy();					   //
} 

//д
void W25QXX_Write_NoCheck(uint32_t Addr,uint8_t* pBuffer,uint32_t Len)   
{ 			 		 
	uint32_t pageremain;	
    

	pageremain=256-(Addr&0xFF); //	    
	if(Len<=pageremain)	pageremain=Len;//
    
//	printf("W25QXX_Write_NoCheck 0x%X:0x%X.pageremain=0x%X\n",Addr,Len,pageremain);    
	while(1)
	{	   
		W25QXX_WritePage(Addr,pBuffer,pageremain);
		if(Len == pageremain)	break;
	 	else
		{
			pBuffer += pageremain;
			Addr += pageremain;	

			Len -= pageremain;			  	//
			if(Len > 256)	pageremain=256; //
			else pageremain = Len; 	  //
		}
	}	    
}  


//д
void W25QXX_Write(uint32_t Addr,uint8_t* pBuffer,uint32_t Len)   
{ 
	uint8_t 	W25QXX_BUFFER[4096];
	uint32_t 	secpos;
	uint32_t 	secoff;
	uint32_t 	secremain;	 
	
 	uint32_t 	i,j;    
	uint8_t  	* W25QXX_BUF;	
	
//                uint8_t * p;
//                printf("W25QXX_Write 0x%X:0x%X.\n",Addr,Len);
//                p = pBuffer;
//                for(i=0; i!=Len>>4; i++)
//                {
//                    for(j=0; j!=16; j++)
//                    {
//                        printf("%02X ",*p++); 
//                    }
//                    printf("\n"); 
//                }
                
                
   	W25QXX_BUF=W25QXX_BUFFER;	
	
 	secpos = Addr&(0xFFFFF000);// 
	secoff = Addr&0xFFF;//
	secremain = 0x1000-secoff;//
	
 	if(Len <= secremain)	secremain = Len;
	while(1) 
	{	
		W25QXX_Read(secpos,W25QXX_BUF,4096);
		for(i=0;i<secremain;i++)
		{
			if(W25QXX_BUF[secoff+i]!=0XFF)break;  
		}
		
		if(i<secremain)//Ҫ
		{
			W25QXX_EraseSector(secpos);		//
			for(i=0;i<secremain;i++)	   		//
			{
				W25QXX_BUF[i+secoff]=pBuffer[i];	  
			}
			W25QXX_Write_NoCheck(secpos,W25QXX_BUF,4096);//

		}else W25QXX_Write_NoCheck(Addr,pBuffer,secremain);//
		
		if(Len==secremain)	break;//
		else
		{
			secpos++;//
			secoff=0;// 

		   	pBuffer += secremain;  				//
			Addr += secremain;				// 
		   	Len  -= secremain;			//
			if(Len>4096)	secremain=4096;//
			else secremain=Len;		//
		}	 
	}	 
}

uint8_t W25QXX_Init(void)
{
    uint32_t id;
    
    Flash_SPI_Configuration();
	id = W25QXX_ReadID();
    if((id == W25Q80) || (id == W25Q16)|| (id == W25Q32)|| (id == W25Q64)|| (id == W25Q128))
    {
        //printf("Flash ID:0x%04X.\n", id);
        W25QXX_WAKEUP();
        return 0;
    }
    return 1;
}

